==> hermes-2.4.12.1.tar.gz: hermes-2.4.12.1/spec/models/housing_request_entity_form_spec.rb <==
require 'rails_helper'

RSpec.describe Hermes::HousingRequestEntityForm, type: :model do
  let!(:housing_request) { create :active_request }

  describe 'for structural tests' do
    it 'has a valid factory' do
      expect(FactoryBot.build(:entity_form)).to be_valid
    end
  end

  describe 'validations' do
    describe 'uniqueness of housing_request_id' do
      let!(:entity_form) { create :entity_form, housing_request_id: housing_request.id }
      let(:new_entity_form) { build :entity_form, housing_request_id: housing_request.id }

      it 'raises error' do
        expect(new_entity_form).to_not be_valid
        expect { new_entity_form.save! }.to raise_error(Mongoid::Errors::Validations, /est déjà pris/)
      end
    end
  end

  describe 'callbacks' do
    let!(:form_level) { create :entity_form_level, level: 4, delegation: true }

    describe "#set_applicant_id" do
      let(:entity_form) { build :entity_form, housing_request_id: housing_request.id }

      it 'sets the applicant_id' do
        expect { entity_form.save }.to change { entity_form.applicant_id }.from(nil).to housing_request.main_applicant_id
      end
    end

    describe "#after_save_actions" do
      let(:entity_form) { build :entity_form, entity_form_level_id: form_level.id }

      it 'sets the entity_id' do
        expect { entity_form.save }.to change { entity_form.entity_id }.from(nil).to form_level.entity_id
      end
      it 'sets the level' do
        expect { entity_form.save }.to change { entity_form.level }.from(nil).to(4)
      end
      it 'sets the delegation' do
        expect { entity_form.save }.to change { entity_form.delegation }.from(false).to(true)
      end

      describe '#update_housing_request_consent_level' do
        context 'when direct set level' do
          let(:entity_form_1) { build :entity_form, housing_request_id: housing_request.id, level: 2 }
          it 'updates the housing_request' do
            expect do
              entity_form_1.save
            end.to change { housing_request.reload.consent_level }.from(nil).to 2
          end
        end
        context 'when level set from entity_form_level' do
          let(:entity_form_2) { build :entity_form, housing_request_id: housing_request.id, entity_form_level_id: form_level.id }
          it 'updates the housing_request' do
            expect do
              entity_form_2.save
            end.to change { housing_request.reload.consent_level }.from(nil).to 4
          end
        end
      end

      describe "#notify_delegation_for_applicant" do
        let!(:housing_request) { create :active_request, :with_applicants }
        before do
          allow(Hermes::HousingRequest).to receive(:find_by).and_return housing_request
        end

        context 'when delegation_signed is false' do
          let(:entity_form) do
            build :entity_form,
              housing_request_id: housing_request.id,
              delegation: true,
              delegation_signed: false,
              entity_form_level_id: form_level.id
          end
          xit 'calls notify on housing_request' do
            expect(housing_request).to receive(:notify).and_call_original
            entity_form.save
          end
          xit 'notifies only the main_applicant' do
            expect { entity_form.save }.to change { NotificationCenter::Notification.count }.by(1)
              .and change { housing_request.main_applicant.notifications.count }.by 1
          end
        end

        context 'when delegation_signed is true' do
          let(:entity_form) do
            build :entity_form,
              housing_request_id: housing_request.id,
              delegation: true,
              delegation_signed: true,
              entity_form_level_id: form_level.id
          end
          it 'does nothing' do
            expect(housing_request).to_not receive(:notify).and_call_original
            entity_form.save
          end
        end
      end
    end
  end
end

==> hermes-6.0.2.29.tar.gz: hermes-6.0.2.29/spec/models/housing_request_entity_form_spec.rb <==
require 'rails_helper'

RSpec.describe Hermes::HousingRequestEntityForm, type: :model do
  let!(:housing_request) { create :active_request }

  describe 'for structural tests' do
    it 'has a valid factory' do
      expect(FactoryBot.build(:entity_form)).to be_valid
    end
  end

  describe 'validations' do
    describe 'uniqueness of housing_request_id' do
      let!(:entity_form) { create :entity_form, housing_request_id: housing_request.id }
      let(:new_entity_form) { build :entity_form, housing_request_id: housing_request.id }

      it 'raises error' do
        expect(new_entity_form).to_not be_valid
        expect { new_entity_form.save! }.to raise_error(Mongoid::Errors::Validations)
      end
    end
  end

  describe 'callbacks' do
    let!(:form_level) { create :entity_form_level, level: 4, delegation: true }

    describe "#set_applicant_id" do
      let(:entity_form) { build :entity_form, housing_request_id: housing_request.id }

      it 'sets the applicant_id' do
        expect { entity_form.save }.to change { entity_form.applicant_id }.from(nil).to housing_request.main_applicant_id
      end
    end

    describe "#after_save_actions" do
      let(:entity_form) { build :entity_form, entity_form_level_id: form_level.id }

      it 'sets the entity_id' do
        expect { entity_form.save }.to change { entity_form.entity_id }.from(nil).to form_level.entity_id
      end
      it 'sets the level' do
        expect { entity_form.save }.to change { entity_form.level }.from(nil).to(4)
      end
      it 'sets the delegation' do
        expect { entity_form.save }.to change { entity_form.delegation }.from(false).to(true)
      end

      describe '#update_housing_request_consent_level' do
        context 'when direct set level' do
          let(:entity_form_1) { build :entity_form, housing_request_id: housing_request.id, level: 2 }
          it 'updates the housing_request' do
            expect do
              entity_form_1.save
            end.to change { housing_request.reload.consent_level }.from(nil).to 2
          end
        end
        context 'when level set from entity_form_level' do
          let(:entity_form_2) { build :entity_form, housing_request_id: housing_request.id, entity_form_level_id: form_level.id }
          it 'updates the housing_request' do
            expect do
              entity_form_2.save
            end.to change { housing_request.reload.consent_level }.from(nil).to 4
          end
        end
      end

      describe "#notify_delegation_for_applicant" do
        let!(:housing_request) { create :active_request, :with_applicants }
        before do
          allow(Hermes::HousingRequest).to receive(:find_by).and_return housing_request
        end

        context 'when delegation_signed is false' do
          let(:entity_form) do
            build :entity_form,
              housing_request_id: housing_request.id,
              delegation: true,
              delegation_signed: false,
              entity_form_level_id: form_level.id
          end
          xit 'calls notify on housing_request' do
            expect(housing_request).to receive(:notify).and_call_original
            entity_form.save
          end
          xit 'notifies only the main_applicant' do
            expect { entity_form.save }.to change { NotificationCenter::Notification.count }.by(1)
              .and change { housing_request.main_applicant.notifications.count }.by 1
          end
        end

        context 'when delegation_signed is true' do
          let(:entity_form) do
            build :entity_form,
              housing_request_id: housing_request.id,
              delegation: true,
              delegation_signed: true,
              entity_form_level_id: form_level.id
          end
          it 'does nothing' do
            expect(housing_request).to_not receive(:notify).and_call_original
            entity_form.save
          end
        end
      end
    end
  end
end
